@@ -1,19 +1,19 @@
-/* A Bison parser, made by GNU Bison 2.5.  */
+/* A Bison parser, made by GNU Bison 3.0.2.  */
 
 /* Bison interface for Yacc-like parsers in C
-   
-      Copyright (C) 1984, 1989-1990, 2000-2011 Free Software Foundation, Inc.
-   
+
+   Copyright (C) 1984, 1989-1990, 2000-2013 Free Software Foundation, Inc.
+
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
@@ -26,27 +26,35 @@
    special exception, which will cause the skeleton and the resulting
    Bison output files to be licensed under the GNU General Public
    License without this special exception.
-   
+
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
+#ifndef YY_YY_PARSER_H_INCLUDED
+# define YY_YY_PARSER_H_INCLUDED
+/* Debug traces.  */
+#ifndef YYDEBUG
+# define YYDEBUG 0
+#endif
+#if YYDEBUG
+extern int yydebug;
+#endif
 
-/* Tokens.  */
+/* Token type.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
-   /* Put the tokens into the symbol table, so that GDB and other debuggers
-      know about them.  */
-   enum yytokentype {
-     NUMBER = 258,
-     BTRUE = 259,
-     BFALSE = 260,
-     QUOTED_STRING = 261,
-     UNQUOTED_STRING = 262,
-     SLASH = 263,
-     OPENBRAC = 264,
-     CLOSEBRAC = 265,
-     EOL = 266
-   };
+  enum yytokentype
+  {
+    NUMBER = 258,
+    BTRUE = 259,
+    BFALSE = 260,
+    QUOTED_STRING = 261,
+    UNQUOTED_STRING = 262,
+    SLASH = 263,
+    OPENBRAC = 264,
+    CLOSEBRAC = 265,
+    EOL = 266
+  };
 #endif
 /* Tokens.  */
 #define NUMBER 258
@@ -59,15 +67,12 @@
 #define CLOSEBRAC 265
 #define EOL 266
 
-
-
-
+/* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
-typedef union YYSTYPE
+typedef union YYSTYPE YYSTYPE;
+union YYSTYPE
 {
-
-/* Line 2068 of yacc.c  */
-#line 38 "parser.y"
+#line 46 "parser.y" /* yacc.c:1909  */
 
 	double  number;
 	int     boolean;
@@ -77,16 +82,15 @@ typedef union YYSTYPE
 	argument_list_t  al;
 	statement_list_t sl;
 
-
-
-/* Line 2068 of yacc.c  */
-#line 84 "parser.h"
-} YYSTYPE;
+#line 86 "parser.h" /* yacc.c:1909  */
+};
 # define YYSTYPE_IS_TRIVIAL 1
-# define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 #endif
 
+
 extern YYSTYPE yylval;
 
+int yyparse (void);
 
+#endif /* !YY_YY_PARSER_H_INCLUDED  */
