@@ -32,6 +32,7 @@
 #define V1STRING "Common Name,Real Address,Bytes Received,Bytes Sent,Connected Since\n"
 #define V2STRING "HEADER,CLIENT_LIST,Common Name,Real Address,Virtual Address,Bytes Received,Bytes Sent,Connected Since,Connected Since (time_t)\n"
 #define V3STRING "HEADER CLIENT_LIST Common Name Real Address Virtual Address Bytes Received Bytes Sent Connected Since Connected Since (time_t)\n"
+#define V4STRING "HEADER,CLIENT_LIST,Common Name,Real Address,Virtual Address,Bytes Received,Bytes Sent,Connected Since,Connected Since (time_t),Username\n"
 #define VSSTRING "OpenVPN STATISTICS\n"
 
 
@@ -43,6 +44,7 @@ struct vpn_status_s
 		MULTI1 = 1, /* status-version 1 */
 		MULTI2,     /* status-version 2 */
 		MULTI3,     /* status-version 3 */
+		MULTI4,     /* status-version 4 */
 		SINGLE = 10 /* currently no versions for single mode, maybe in the future */
 	} version;
 	char *name;
@@ -265,7 +267,7 @@ static int multi1_read (char *name, FILE *fh)
 {
 	char buffer[1024];
 	char *fields[10];
-	int  fields_num, read = 0, found_header = 0;
+	int  fields_num, found_header = 0;
 	long long sum_users = 0;
 
 	/* read the file until the "ROUTING TABLE" line is found (no more info after) */
@@ -312,17 +314,15 @@ static int multi1_read (char *name, FILE *fh)
 						atoll (fields[3])); /* "Bytes Sent" */
 			}
 		}
-
-		read = 1;
 	}
 
+	if (ferror (fh))
+		return (0);
+
 	if (collect_user_count)
-	{
 		numusers_submit(name, name, sum_users);
-		read = 1;
-	}
 
-	return (read);
+	return (1);
 } /* int multi1_read */
 
 /* for reading status version 2 */
@@ -452,13 +452,77 @@ static int multi3_read (char *name, FILE *fh)
 	return (read);
 } /* int multi3_read */
 
+/* for reading status version 4 */
+static int multi4_read (char *name, FILE *fh)
+{
+	char buffer[1024];
+	char *fields[11];
+	const int max_fields = STATIC_ARRAY_SIZE (fields);
+	int  fields_num, read = 0;
+	long long sum_users    = 0;
+
+	while (fgets (buffer, sizeof (buffer), fh) != NULL)
+	{
+		fields_num = openvpn_strsplit (buffer, fields, max_fields);
+
+		/* status file is generated by openvpn/multi.c:multi_print_status()
+		 * http://svn.openvpn.net/projects/openvpn/trunk/openvpn/multi.c
+		 *
+		 * The line we're expecting has 9 fields. We ignore all lines
+		 *  with more or less fields.
+		 */
+		if (fields_num != 9)
+			continue;
+
+
+		if (strcmp (fields[0], "CLIENT_LIST") != 0)
+			continue;
+
+
+		if (collect_user_count)
+			/* If so, sum all users, ignore the individuals*/
+		{
+			sum_users += 1;
+		}
+		if (collect_individual_users)
+		{
+			if (new_naming_schema)
+			{
+				/* plugin inst = file name, type inst = fields[1] */
+				iostats_submit (name,               /* vpn instance */
+						fields[1],          /* "Common Name" */
+						atoll (fields[4]),  /* "Bytes Received" */
+						atoll (fields[5])); /* "Bytes Sent" */
+			}
+			else
+			{
+				/* plugin inst = fields[1], type inst = "" */
+				iostats_submit (fields[1],          /* "Common Name" */
+						NULL,               /* unused when in multimode */
+						atoll (fields[4]),  /* "Bytes Received" */
+						atoll (fields[5])); /* "Bytes Sent" */
+			}
+		}
+
+		read = 1;
+	}
+
+	if (collect_user_count)
+	{
+		numusers_submit(name, name, sum_users);
+		read = 1;
+	}
+
+	return (read);
+} /* int multi4_read */
+
 /* read callback */
 static int openvpn_read (void)
 {
 	FILE *fh;
-	int  i, read;
+	int  i, vpn_read, read;
 
-	read = 0;
+	vpn_read = read = 0;
 
 	/* call the right read function for every status entry in the list */
 	for (i = 0; i < vpn_num; i++)
@@ -476,23 +540,28 @@ static int openvpn_read (void)
 		switch (vpn_list[i]->version)
 		{
 			case SINGLE:
-				read = single_read(vpn_list[i]->name, fh);
+				vpn_read = single_read(vpn_list[i]->name, fh);
 				break;
 
 			case MULTI1:
-				read = multi1_read(vpn_list[i]->name, fh);
+				vpn_read = multi1_read(vpn_list[i]->name, fh);
 				break;
 
 			case MULTI2:
-				read = multi2_read(vpn_list[i]->name, fh);
+				vpn_read = multi2_read(vpn_list[i]->name, fh);
 				break;
 
 			case MULTI3:
-				read = multi3_read(vpn_list[i]->name, fh);
+				vpn_read = multi3_read(vpn_list[i]->name, fh);
+				break;
+
+			case MULTI4:
+				vpn_read = multi4_read(vpn_list[i]->name, fh);
 				break;
 		}
 
 		fclose (fh);
+		read += vpn_read;
 	}
 
 	return (read ? 0 : -1);
@@ -549,6 +618,13 @@ static int version_detect (const char *filename)
 			version = MULTI3;
 			break;
 		}
+		/* searching for multi version 4 */
+		else if (strcmp (buffer, V4STRING) == 0)
+		{
+			DEBUG ("openvpn plugin: found status file version MULTI4");
+			version = MULTI4;
+			break;
+		}
 	}
 
 	if (version == 0)
